import java.awt.*;
import java.io.*;


class PolygonShape extends Shape {
	
	Polygon polygon;
	boolean fill;
	

	public PolygonShape(Sheet s, Polygon polygon, Color c, boolean fill) {
		
		this.s = s;
		color = c;		
		this.polygon = polygon;
		this.fill = fill;
		createBoundingBox();
		
	}


	public PolygonShape(Sheet s, BufferedReader in) {
		
		this.s = s;
		color = InOut.readColor(in);
		polygon = InOut.readPolygon(in);
		fill = InOut.readBoolean(in);
		
		createBoundingBox();

		try {
			in.readLine();	// skip blank line
		}
    catch (IOException e) {
      System.out.println("IO Error:" + e.getMessage());
    }
    
	}


	void createBoundingBox() {

		int minX = polygon.xpoints[0];
		int minY = polygon.ypoints[0];
		int maxX = minX;
		int maxY = minY;

		for (int i = 1 ; i < polygon.npoints ; i++) {
			minX = Math.min(minX, polygon.xpoints[i]);
			minY = Math.min(minY, polygon.ypoints[i]);
			maxX = Math.max(maxX, polygon.xpoints[i]);
			maxY = Math.max(maxY, polygon.ypoints[i]);
		}

		boundingBox = Geometry.createRectangle(new Point(minX, minY), new Point(maxX, maxY));
	}

	
	protected Object clone() {
		
		Polygon p = new Polygon(polygon.xpoints, polygon.ypoints, polygon.npoints);
		return new PolygonShape(s, p, color, fill);
		
	}

	
	public String toString() {
		
		String s;

		s = "Polygon\n";
		s = s + InOut.writeColor(color);
		s = s + InOut.writePolygon(polygon);
		s = s + InOut.writeBoolean(fill) + "\n";
		
		return s;
		
	}

	
	void draw(Graphics g) {
		
		g.setColor(color);

		if (boundingBox.isEmpty()) {
			Point o = Geometry.opposite(new Point(boundingBox.x, boundingBox.y), boundingBox);
			g.drawLine(boundingBox.x, boundingBox.y, o.x, o.y);
		}
		else {
			g.drawPolygon(polygon);
			if (fill) {
				g.fillPolygon(polygon);
			}
		}
		
	}


	boolean contains(int x, int y) {

		Point p = new Point(x, y);
		
		if (boundingBox.isEmpty()) {
			Point p1 = new Point(boundingBox.x, boundingBox.y);
			Point p2 = Geometry.opposite(p1, boundingBox);
			return Geometry.segmentContains(p1, p2, p);
		}
		else if (fill) {
			return polygon.contains(p);
		}
		else {
			for (int i = 0 ; i < polygon.npoints - 1 ; i++) {
				Point p1 = new Point(polygon.xpoints[i], polygon.ypoints[i]);
				Point p2 = new Point(polygon.xpoints[i + 1], polygon.ypoints[i + 1]);
				if (Geometry.segmentContains(p1, p2, p)) {
					return true;
				}
			}

			Point p1 = new Point(polygon.xpoints[polygon.npoints - 1],
													 polygon.ypoints[polygon.npoints - 1]);
			Point p2 = new Point(polygon.xpoints[0], polygon.ypoints[0]);
			return Geometry.segmentContains(p1, p2, p);			 
		}
		
	}


	void translate(int dx, int dy) {

		boundingBox.translate(dx, dy);

		polygon.translate(dx, dy);

		if (selected) {
			removeHandles();
			createHandles();
		}
		
	}


	void rotate(Point c) {

		for (int i = 0 ; i < polygon.npoints ; i++) {
			Point p1 = new Point(polygon.xpoints[i], polygon.ypoints[i]);
			Point p2 = Geometry.rotate90CW(p1, c);
			
			polygon.xpoints[i] = p2.x;
			polygon.ypoints[i] = p2.y;
		}

		// polygon.contains(p) uses getBounds(), so this other bounding box needs to
		// be kept up to date:
		polygon = new Polygon(polygon.xpoints, polygon.ypoints, polygon.npoints);
		
		createBoundingBox();
				
	}


	void scale(Shape initialShape, Point o, float xScale, float yScale) {

		for (int i = 0 ; i < polygon.npoints ; i++) {
			Point p1 = new Point(((PolygonShape) initialShape).polygon.xpoints[i],
													 ((PolygonShape) initialShape).polygon.ypoints[i]);
			Point p2 = Geometry.scale(p1, o, xScale, yScale);
			
			polygon.xpoints[i] = p2.x;
			polygon.ypoints[i] = p2.y;
		}

		// polygon.contains(p) uses getBounds(), so this other bounding box needs to
		// be kept up to date:
		polygon = new Polygon(polygon.xpoints, polygon.ypoints, polygon.npoints);
		
		createBoundingBox();
		
	}

	
	void reshape(Shape initialShape, Point start, Point end) {

		Point o = Geometry.opposite(start, initialShape.boundingBox);
				
		float xRatio;
		if (start.x == o.x) {
			xRatio = 1;
		}
		else {
			xRatio = ((float) (end.x - o.x)) / (start.x - o.x);
		}
		
		float yRatio;
		if (start.y == o.y) {
			yRatio = 1;
		}
		else {
			yRatio = ((float) (end.y - o.y)) / (start.y - o.y);
		}
		
		scale(initialShape, o, xRatio, yRatio);

	}


	void createExtendedHandles() {
		
		for (int i = 0 ; i < polygon.npoints ; i++) {
			Point p = new Point(polygon.xpoints[i], polygon.ypoints[i]);
			s.handleList.addElement(new Handle(this, p));
		}

	}


	void extendedEdit(Point previousPosition, Point p) {
		
		for (int i = polygon.npoints - 1 ; i >= 0 ; i--) {
			if ((polygon.xpoints[i] == previousPosition.x) &&
					(polygon.ypoints[i] == previousPosition.y)) {
						
				polygon.xpoints[i] = p.x;
				polygon.ypoints[i] = p.y;
				
				polygon = new Polygon(polygon.xpoints, polygon.ypoints, polygon.npoints);
				createBoundingBox();
				
				break;
			}
		}

	}
	
}

